@@ -1,13 +1,13 @@
 "writeTiff" <-
 function(pixmap, fn) {
-      if(class(pixmap) == "pixmapRGB"){
+      if(inherits(pixmap, "pixmapRGB")) {
         .Call("C_writeTiff", pixmap@red, pixmap@green, pixmap@blue, fn)
-      } else if(class(pixmap) == "matrix") {
+      } else if(inherits(pixmap, "matrix")) {
         pixmap = newPixmapRGB(pixmap, pixmap, pixmap);
         .Call("C_writeTiff", pixmap@red, pixmap@green, pixmap@blue, fn)
       } else {
         stop(paste("writeTiff expects a pixmapRGB or matrix, got ", class(pixmap)))
       }
-      gc();
-      return();
+      gc()
+      invisible(NULL)
 }
