@@ -4,7 +4,7 @@
 #include <Rdefines.h>
 #include <Rinternals.h>
 
-/*============================================================================ 
+/*============================================================================
  * tiffRead --
  *
  * Copyleft 2001 by Eric J. Kort/Van Andel Research Institute
@@ -15,7 +15,7 @@
  * Copyright (c) 1988-1997 Sam Leffler
  * Copyright (c) 1991-1997 Silicon Graphics, Inc.
  *
- * Permission to use, copy, modify, distribute, and sell this software and 
+ * Permission to use, copy, modify, distribute, and sell this software and
  * its documentation for any purpose is hereby granted without fee, provided
  * that (i) the above copyright notices and this permission notice appear in
  * all copies of the software and related documentation, and (ii) the names of
@@ -25,7 +25,7 @@
  *============================================================================
  */
 
-void TiffGetHeight (char** filename, int* h) 
+void TiffGetHeight (char** filename, int* h)
 {
     TIFF* tif;
 
@@ -51,7 +51,7 @@ void TiffGetWidth (char** filename, int* w)
 	*w = -1;
 	return;
     }
-    TIFFGetField(tif, TIFFTAG_IMAGEWIDTH, w);    
+    TIFFGetField(tif, TIFFTAG_IMAGEWIDTH, w);
     TIFFClose(tif);
     return;
 }
@@ -66,7 +66,7 @@ void TiffIsTiled(char **filename, int* yn)
     return;
 }
 
-void TiffGetImageType (char** filename, int *dir, int* spp, int* pm, int* bps, int* tiled) 
+void TiffGetImageType (char** filename, int *dir, int* spp, int* pm, int* bps, int* tiled)
 {
     TIFF* tif;
 
@@ -78,7 +78,7 @@ void TiffGetImageType (char** filename, int *dir, int* spp, int* pm, int* bps, i
     }
 
     TIFFGetField(tif, TIFFTAG_PHOTOMETRIC, pm);
-    TIFFGetField(tif, TIFFTAG_SAMPLESPERPIXEL, spp);        
+    TIFFGetField(tif, TIFFTAG_SAMPLESPERPIXEL, spp);
     TIFFGetField(tif, TIFFTAG_BITSPERSAMPLE, bps);
     *tiled = TIFFIsTiled(tif);
     TIFFClose(tif);
@@ -111,10 +111,10 @@ void TiffReadTIFFRGBA (char** filename, int* dir, int* r, int* g, int* b)
 		r[i] = (int)TIFFGetR(buf[i]);
 		g[i] = (int)TIFFGetG(buf[i]);
 		b[i] = (int)TIFFGetB(buf[i]);
-	    }                
+	    }
 	}
     } else
-	error("Error allocating memory in TIFFmalloc");  
+	error("Error allocating memory in TIFFmalloc");
     TIFFClose(tif);
     _TIFFfree(buf);
 }
@@ -136,7 +136,7 @@ SEXP getTiffDescription(SEXP fn)
     return ans;
 }
 
-void writeTiff(SEXP mr, SEXP mg, SEXP mb, SEXP fn)
+SEXP writeTiff(SEXP mr, SEXP mg, SEXP mb, SEXP fn)
 {
     TIFF *output;
     unsigned char *raster;
@@ -147,6 +147,7 @@ void writeTiff(SEXP mr, SEXP mg, SEXP mb, SEXP fn)
     double *g = REAL(mg);
     double *b = REAL(mb);
     const char* filename = CHAR(STRING_ELT(fn, 0));
+    SEXP rv = R_NilValue;
 
     // Open the output image
     if((output = TIFFOpen(filename, "w")) == NULL)
@@ -181,9 +182,10 @@ void writeTiff(SEXP mr, SEXP mg, SEXP mb, SEXP fn)
 
     TIFFClose(output);
     free(raster);
+    return(rv);
 }
 
-void reduce(int* r, int* nr, int* w, int* h, double* p) 
+void reduce(int* r, int* nr, int* w, int* h, double* p)
 {
     int nw, nh, x, y, i;
     nw = ceil((1-*p) * *w);
@@ -197,25 +199,48 @@ void reduce(int* r, int* nr, int* w, int* h, double* p)
 }
 
 
-void updateTTag (SEXP fn, SEXP desc)
+SEXP updateTTag (SEXP fn, SEXP desc)
 {
-#if TIFF_VERSION_CLASSIC >= 40
-    error("Interface changed in libtiff 4, so no longer implemented");
-#else
     TIFF *tiff;
+    SEXP rv = R_NilValue;
+
     const char* filename = CHAR(STRING_ELT(fn, 0)) ;
-    const char* description = CHAR(STRING_ELT(desc, 0)) ;	
+    const char* description = CHAR(STRING_ELT(desc, 0)) ;
     if((tiff = TIFFOpen(filename , "r+")) == NULL)
-	error("Could not open image file '%s'", filename);
+    	error("Could not open image file '%s'", filename);
+#if TIFF_VERSION_CLASSIC >= 40
+    const TIFFField *fip;
+#else
     const TIFFFieldInfo *fip;
+#endif
     fip = TIFFFieldWithTag(tiff, 270);
     if (!fip) error("Could not get field information");
-    if (fip->field_type == TIFF_ASCII) {
-        if (TIFFSetField(tiff, fip->field_tag, description) != 1)
-            error("Failed to set field.");
-    } else error("Description field is not ascii");
+    if (TIFFSetField(tiff, TIFFTAG_IMAGEDESCRIPTION, description) != 1)
+        error("Failed to set field.");
     TIFFRewriteDirectory(tiff);
     TIFFClose(tiff);
-#endif
+    return(rv);
+}
+
+
+static const R_CMethodDef CEntries[] = {
+  {"C_reduce",           (DL_FUNC) &reduce,           5},
+  {"C_TiffGetHeight",    (DL_FUNC) &TiffGetHeight,    2},
+  {"C_TiffGetWidth",     (DL_FUNC) &TiffGetWidth,     2},
+  {"C_TiffReadTIFFRGBA", (DL_FUNC) &TiffReadTIFFRGBA, 5},
+  {NULL, NULL, 0}
+};
+
+static const R_CallMethodDef CallEntries[] = {
+  {"C_getTiffDescription", (DL_FUNC) &getTiffDescription, 1},
+  {"C_updateTTag",         (DL_FUNC) &updateTTag,         2},
+  {"C_writeTiff",          (DL_FUNC) &writeTiff,          4},
+  {NULL, NULL, 0}
+};
+
+void R_init_rtiff(DllInfo *dll)
+{
+  R_registerRoutines(dll, CEntries, CallEntries, NULL, NULL);
+  R_useDynamicSymbols(dll, FALSE);
 }
 
